import 'package:flutter_mylibrary/base/request_info.dart';
import 'package:flutter_mylibrary/config/const.dart';
import 'package:flutter_mylibrary/flutter_mylibrary.dart';
import 'package:flutter_mylibrary/http/easy_dio.dart';
import 'package:flutter_mylibrary/http/exception/http_error.dart';
import 'package:flutter_mylibrary/http/http_data.dart';
import 'package:flutter_mylibrary/utils/utils.dart';

abstract class BaseViewState{

  /// 防止页面销毁后,异步任务才完成,导致报错
  bool disposed = false;

  /// 当前的页面状态,默认为busy,可在viewModel的构造方法中指定;
  ViewState viewState = ViewState.busy;

  EasyRefreshController _refreshController = EasyRefreshController();

  set refreshController(EasyRefreshController easyRefreshController){
    this._refreshController = easyRefreshController;
  }

  ///获取请求的数据，一般用于简单的页面请求
  RequestInfo getRequestInfo();

  RequestInfo beforeRequest(RequestInfo requestInfo){
    return requestInfo;
  }

  RequestInfo requestInfo;

  void doOther(){}

  EasyRefreshController get refreshController => _refreshController;

  bool isInit = false;


  ///重新请求当报错时
  void requestWhenError();

  ///---------------------------------------------页面状态------------------------------------------------------

  HttpError httpError;

  bool get isBusy => viewState == ViewState.busy;

  bool get isIdle => viewState == ViewState.idle;

  bool get isError => viewState == ViewState.error;


  ///---------------------------------------------设置页面状态------------------------------------------------------


  void setIdle();

  void setBusy();

  void setError(HttpError httpError);



  void onError(HttpError httpError) {}


  ///---------------------------------------------通用请求方法------------------------------------------------------

  ///通常有下拉刷新请求，第一页之后不会再设置页面错误状态
  Future<HttpData> request(void handleHttpData(HttpData data), {bool isSetError = true}) async{

    requestInfo = beforeRequest(requestInfo??getRequestInfo());

    doOther();

    HttpData _httpData = await doRequest(requestInfo);

    var data = _httpData.data;

    if(isEmpty(_httpData.httpError)){
      if(isNotEmpty(data)){
        handleHttpData(_httpData);
      }else{
        _httpData.httpError = HttpError.emptyDataError(data);
      }
      return _httpData;
    }else{
      if(isSetError){
        setError(_httpData.httpError);
      }
      return _httpData;
    }
  }


  Future<HttpData> doRequest(RequestInfo requestInfo) async{
    HttpData httpData = await EasyDio().request(
      requestInfo?.method,
      requestInfo?.url,
      data: requestInfo?.data,
      publicMethod: requestInfo?.publicMethod,
    );

    try{
      var data = httpData.data;
      if(data is List){
        httpData.data = data.map((e) => requestInfo?.handleRes(e));
      }else if(data is Map){
        httpData.data = requestInfo?.handleRes(data);
      }
      if(isNotEmpty(requestInfo.validateError) && isEmpty(httpData.httpError)){
        httpData.httpError = requestInfo.validateError(httpData.data);
      }
    }catch(e){
      httpData.httpError =  HttpError.parseDataError(e);
    }


    return httpData;
  }


}



/// 页面状态类型
enum ViewState {
  idle,
  busy,
  error
}